@@ -3,16 +3,8 @@
 typedef void (request_func_t) (struct scsi_cmd *);
 
 struct bs_thread_info {
-	pthread_t ack_thread;
 	pthread_t worker_thread[NR_WORKER_THREADS];
 
-	/* protected by pipe */
-	struct list_head ack_list;
-
-	pthread_cond_t finished_cond;
-	pthread_mutex_t finished_lock;
-	struct list_head finished_list;
-
 	/* wokers sleep on this and signaled by tgtd */
 	pthread_cond_t pending_cond;
 	/* locked by tgtd and workers */
@@ -22,9 +14,6 @@ struct bs_thread_info {
 
 	pthread_mutex_t startup_lock;
 
-	int command_fd[2];
-	int done_fd[2];
-
 	int stop;
 
 	request_func_t *request_fn;
