@@ -31,7 +31,6 @@
 #include <netinet/tcp.h>
 #include <sys/epoll.h>
 #include <sys/socket.h>
-#include <pthread.h>
 
 #include "iscsid.h"
 #include "tgtd.h"
@@ -44,7 +43,6 @@ static struct iscsi_transport iscsi_tcp;
 
 struct iscsi_tcp_connection {
 	int fd;
-	int pthread;
 
 	struct iscsi_connection iscsi_conn;
 };
@@ -155,7 +153,6 @@ out:
 static void iscsi_tcp_event_handler(int fd, int events, void *data)
 {
 	struct iscsi_connection *conn = (struct iscsi_connection *) data;
-	struct iscsi_tcp_connection *tcp_conn = TCP_CONN(conn);
 
 	if (events & EPOLLIN)
 		iscsi_rx_handler(conn);
@@ -168,36 +165,29 @@ static void iscsi_tcp_event_handler(int fd, int events, void *data)
 
 	if (conn->state == STATE_CLOSE) {
 		dprintf("connection closed %p\n", conn);
-		if (tcp_conn->pthread) {
-			struct iscsi_target *target = conn->session->target;
-
-			pthread_mutex_lock(&target->event_lock);
-			do_tgt_event_del(target->efd, &target->events_list,
-					 tcp_conn->fd);
-			pthread_mutex_unlock(&target->event_lock);
-			/* let the main thread handle this */
-			tcp_conn->pthread = 0;
-			tgt_event_modify(tcp_conn->fd, EPOLLIN|EPOLLOUT|EPOLLERR);
-		} else {
-			conn_close(conn);
-		}
+		conn_close(conn);
 	}
 }
 
-static int iscsi_tcp_init(void)
+int iscsi_tcp_init_portal(char *addr, int port, int tpgt)
 {
 	struct addrinfo hints, *res, *res0;
 	char servname[64];
 	int ret, fd, opt, nr_sock = 0;
+	struct iscsi_portal *portal = NULL;
+	char addrstr[64];
+	void *addrptr;
+
+	port = port ? port : ISCSI_LISTEN_PORT;
 
 	memset(servname, 0, sizeof(servname));
-	snprintf(servname, sizeof(servname), "%d", iscsi_listen_port);
+	snprintf(servname, sizeof(servname), "%d", port);
 
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_flags = AI_PASSIVE;
 
-	ret = getaddrinfo(iscsi_portal_addr, servname, &hints, &res0);
+	ret = getaddrinfo(addr, servname, &hints, &res0);
 	if (ret) {
 		eprintf("unable to get address info, %m\n");
 		return -errno;
@@ -253,6 +243,25 @@ static int iscsi_tcp_init(void)
 			listen_fds[nr_sock] = fd;
 			nr_sock++;
 		}
+
+		portal = zalloc(sizeof(struct iscsi_portal));
+		switch (res->ai_family) {
+		case AF_INET:
+			addrptr = &((struct sockaddr_in *)
+				    res->ai_addr)->sin_addr;
+			break;
+		case AF_INET6:
+			addrptr = &((struct sockaddr_in6 *)
+				    res->ai_addr)->sin6_addr;
+			break;
+		}
+		portal->addr = strdup(inet_ntop(res->ai_family, addrptr,
+			     addrstr, sizeof(addrstr)));
+		portal->port = port;
+		portal->tpgt = tpgt;
+		portal->fd   = fd;
+
+		list_add(&portal->iscsi_portal_siblings, &iscsi_portals_list);
 	}
 
 	freeaddrinfo(res0);
@@ -260,6 +269,53 @@ static int iscsi_tcp_init(void)
 	return !nr_sock;
 }
 
+int iscsi_add_portal(char *addr, int port, int tpgt)
+{
+	if (iscsi_tcp_init_portal(addr, port, tpgt)) {
+		eprintf("failed to create/bind to portal %s:%d\n", addr, port);
+		return -1;
+	}
+
+	return 0;
+};
+
+int iscsi_delete_portal(char *addr, int port)
+{
+	struct iscsi_portal *portal;
+
+	list_for_each_entry(portal, &iscsi_portals_list,
+			    iscsi_portal_siblings) {
+		if (!strcmp(addr, portal->addr) && port == portal->port) {
+			if (portal->fd != -1)
+				tgt_event_del(portal->fd);
+			close(portal->fd);
+			list_del(&portal->iscsi_portal_siblings);
+			free(portal->addr);
+			free(portal);
+			return 0;
+		}
+	}
+	eprintf("delete_portal failed. No such portal found %s:%d\n",
+			addr, port);
+	return -1;
+}
+
+static int iscsi_tcp_init(void)
+{
+	/* If we were passed any portals on the command line */
+	if (portal_arguments)
+		iscsi_param_parse_portals(portal_arguments, 1, 0);
+
+	/* if the user did not set a portal we default to wildcard
+	   for ipv4 and ipv6
+	*/
+	if (list_empty(&iscsi_portals_list)) {
+		iscsi_add_portal(NULL, 3260, 1);
+	}
+
+	return 0;
+}
+
 static void iscsi_tcp_exit(void)
 {
 	int i;
@@ -278,29 +334,6 @@ static int iscsi_tcp_conn_login_complete(struct iscsi_connection *conn)
 	return 0;
 }
 
-static void iscsi_tcp_conn_nexus_init(struct iscsi_connection *conn)
-{
-	struct iscsi_tcp_connection *tcp_conn = TCP_CONN(conn);
-	struct iscsi_target *target = conn->session->target;
-
-	if (iscsi_pthread_per_target()) {
-		/* remove the conn from the main thread. */
-		conn->tp->ep_event_modify(conn, 0);
-
-		pthread_mutex_lock(&target->event_lock);
-
-		do_tgt_event_add(target->efd, &target->events_list,
-				 tcp_conn->fd, EPOLLIN,
-				 iscsi_tcp_event_handler, conn);
-
-		pthread_mutex_unlock(&target->event_lock);
-
-		tcp_conn->pthread = 1;
-	}
-
-	conn->tp->ep_event_modify(conn, EPOLLIN);
-}
-
 static size_t iscsi_tcp_read(struct iscsi_connection *conn, void *buf,
 			     size_t nbytes)
 {
@@ -331,7 +364,7 @@ static size_t iscsi_tcp_close(struct iscsi_connection *conn)
 	struct iscsi_tcp_connection *tcp_conn = TCP_CONN(conn);
 
 	tgt_event_del(tcp_conn->fd);
-	return close(tcp_conn->fd);
+	return 0;
 }
 
 static void iscsi_tcp_release(struct iscsi_connection *conn)
@@ -339,6 +372,7 @@ static void iscsi_tcp_release(struct iscsi_connection *conn)
 	struct iscsi_tcp_connection *tcp_conn = TCP_CONN(conn);
 
 	conn_exit(conn);
+	close(tcp_conn->fd);
 	free(tcp_conn);
 }
 
@@ -374,18 +408,9 @@ static void iscsi_event_modify(struct iscsi_connection *conn, int events)
 	struct iscsi_tcp_connection *tcp_conn = TCP_CONN(conn);
 	int ret;
 
-	if (tcp_conn->pthread) {
-		struct iscsi_target *target = conn->session->target;
-
-		pthread_mutex_lock(&target->event_lock);
-		do_tgt_event_modify(target->efd, &target->events_list,
-				    tcp_conn->fd, events);
-		pthread_mutex_unlock(&target->event_lock);
-	} else {
-		ret = tgt_event_modify(tcp_conn->fd, events);
-		if (ret)
-			eprintf("tgt_event_modify failed\n");
-	}
+	ret = tgt_event_modify(tcp_conn->fd, events);
+	if (ret)
+		eprintf("tgt_event_modify failed\n");
 }
 
 static struct iscsi_task *iscsi_tcp_alloc_task(struct iscsi_connection *conn,
@@ -431,6 +456,12 @@ static int iscsi_tcp_getpeername(struct iscsi_connection *conn,
 	return getpeername(tcp_conn->fd, sa, len);
 }
 
+static void iscsi_tcp_conn_force_close(struct iscsi_connection *conn)
+{
+	conn->state = STATE_CLOSE;
+	conn->tp->ep_event_modify(conn, EPOLLIN|EPOLLOUT|EPOLLERR);
+}
+
 static struct iscsi_transport iscsi_tcp = {
 	.name			= "iscsi",
 	.rdma			= 0,
@@ -438,13 +469,13 @@ static struct iscsi_transport iscsi_tcp = {
 	.ep_init		= iscsi_tcp_init,
 	.ep_exit		= iscsi_tcp_exit,
 	.ep_login_complete	= iscsi_tcp_conn_login_complete,
-	.ep_nexus_init		= iscsi_tcp_conn_nexus_init,
 	.alloc_task		= iscsi_tcp_alloc_task,
 	.free_task		= iscsi_tcp_free_task,
 	.ep_read		= iscsi_tcp_read,
 	.ep_write_begin		= iscsi_tcp_write_begin,
 	.ep_write_end		= iscsi_tcp_write_end,
 	.ep_close		= iscsi_tcp_close,
+	.ep_force_close		= iscsi_tcp_conn_force_close,
 	.ep_release		= iscsi_tcp_release,
 	.ep_show		= iscsi_tcp_show,
 	.ep_event_modify	= iscsi_event_modify,
