@@ -43,8 +43,9 @@
 
 #define MAX_QUEUE_CMD	128
 
-int iscsi_listen_port = ISCSI_LISTEN_PORT;
-char *iscsi_portal_addr;
+LIST_HEAD(iscsi_portals_list);
+
+char *portal_arguments;
 
 enum {
 	IOSTATE_FREE,
@@ -827,7 +828,7 @@ static void text_scan_text(struct iscsi_connection *conn)
 			struct sockaddr_storage ss;
 			socklen_t slen, blen;
 			char *p, buf[NI_MAXHOST + 128];
-			int ret;
+			int ret, port;
 
 			if (value[0] == 0)
 				continue;
@@ -861,7 +862,14 @@ static void text_scan_text(struct iscsi_connection *conn)
 			if (ss.ss_family == AF_INET6)
 				 *p++ = ']';
 
-			sprintf(p, ":%d,1", iscsi_listen_port);
+			if (ss.ss_family == AF_INET6)
+				port = ntohs(((struct sockaddr_in6 *)
+						&ss)->sin6_port);
+			else
+				port = ntohs(((struct sockaddr_in *)
+						&ss)->sin_port);
+
+			sprintf(p, ":%d,1", port);
 			target_list_build(conn, buf,
 					  strcmp(value, "All") ? value : NULL);
 		} else
@@ -1425,7 +1433,7 @@ static int iscsi_tm_execute(struct iscsi_task *task)
 		ret = target_mgmt_request(conn->session->target->tid,
 					  conn->session->tsih,
 					  (unsigned long)task, fn, req->lun,
-					  req->itt, 0);
+					  req->rtt, 0);
 		set_task_in_scsi(task);
 		switch (ret) {
 		case MGMT_REQ_QUEUED:
@@ -1987,6 +1995,13 @@ again:
 		conn->req.ahssize = conn->req.bhs.hlength * 4;
 		conn->req.datasize = ntoh24(conn->req.bhs.dlength);
 		conn->rx_size = conn->req.ahssize;
+
+		if (conn->state != STATE_SCSI &&
+		    conn->req.ahssize > INCOMING_BUFSIZE) {
+			conn->state = STATE_CLOSE;
+			return;
+		}
+
 		if (conn->rx_size) {
 			conn->rx_buffer = conn->req.ahs;
 			conn->rx_iostate = IOSTATE_RX_AHS;
@@ -2039,6 +2054,14 @@ again:
 		if (conn->rx_size) {
 			conn->rx_iostate = IOSTATE_RX_DATA;
 			conn->rx_buffer = conn->req.data;
+
+			if (conn->state != STATE_SCSI) {
+				if (conn->req.ahssize + conn->rx_size >
+				    INCOMING_BUFSIZE) {
+					conn->state = STATE_CLOSE;
+					return;
+				}
+			}
 		} else {
 			conn->rx_iostate = IOSTATE_RX_END;
 			break;
@@ -2338,49 +2361,58 @@ int iscsi_transportid(int tid, uint64_t itn_id, char *buf, int size)
 	return len;
 }
 
-static struct tgt_driver iscsi = {
-	.name			= "iscsi",
-	.use_kernel		= 0,
-	.init			= iscsi_init,
-	.exit			= iscsi_exit,
-	.target_create		= iscsi_target_create,
-	.target_destroy		= iscsi_target_destroy,
-
-	.update			= iscsi_target_update,
-	.show			= iscsi_target_show,
-	.cmd_end_notify		= iscsi_scsi_cmd_done,
-	.mgmt_end_notify	= iscsi_tm_done,
-	.transportid		= iscsi_transportid,
-	.default_bst		= "rdwr",
-};
-
-static int iscsi_param_parser(char *p)
+int iscsi_param_parse_portals(char *p, int do_add,
+			int do_delete)
 {
 	while (*p) {
 		if (!strncmp(p, "portal", 6)) {
 			char *addr, *q;
-			int len;
+			int len = 0, port = 0;
 
 			addr = p + 7;
 
-			q = strchr(addr, ':');
+			if (addr[0] == '[') {
+				addr++;
+				q = strchr(addr, ']');
+				if (!q) {
+					eprintf("malformed string when parsing "
+						"portal (%s). mismatched ipv6 "
+						"'[' ']'\n", p);
+					return -1;
+				}
+				q++;
+				len = q - addr -1;
+				if (*q != ':')
+					q = NULL;
+			} else
+				q = strchr(addr, ':');
+
 			if (q)
-				iscsi_listen_port = atoi(q + 1);
+				port = atoi(q + 1);
 			else
 				q = strchr(addr, ',');
 
-			if (q)
-				len = q - addr;
-			else
-				len = strlen(addr);
-
-			if (iscsi_portal_addr) {
-				free(iscsi_portal_addr);
-				iscsi_portal_addr = NULL;
+			if (!len) {
+				if (q)
+					len = q - addr;
+				else
+					len = strlen(addr);
 			}
+
 			if (len) {
-				iscsi_portal_addr = zalloc(len + 1);
-				memcpy(iscsi_portal_addr, addr, len);
+				char *tmp;
+				tmp = zalloc(len + 1);
+				memcpy(tmp, addr, len);
+				if (do_add && iscsi_add_portal(tmp,
+							port, 1)) {
+					free(tmp);
+					return -1;
+				}
+				if (do_delete && iscsi_delete_portal(tmp,
+							port)) {
+					free(tmp);
+					return -1;
+				}
 			}
 		}
 
@@ -2392,6 +2424,41 @@ static int iscsi_param_parser(char *p)
 	return 0;
 }
 
+static int iscsi_param_parser(char *p)
+{
+	portal_arguments = p;
+	return 0;
+}
+
+static int iscsi_portal_create(char *p)
+{
+	return iscsi_param_parse_portals(p, 1, 0);
+}
+
+static int iscsi_portal_destroy(char *p)
+{
+	return iscsi_param_parse_portals(p, 0, 1);
+}
+
+static struct tgt_driver iscsi = {
+	.name			= "iscsi",
+	.use_kernel		= 0,
+	.init			= iscsi_init,
+	.exit			= iscsi_exit,
+	.target_create		= iscsi_target_create,
+	.target_destroy		= iscsi_target_destroy,
+
+	.portal_create		= iscsi_portal_create,
+	.portal_destroy		= iscsi_portal_destroy,
+
+	.update			= iscsi_target_update,
+	.show			= iscsi_target_show,
+	.cmd_end_notify		= iscsi_scsi_cmd_done,
+	.mgmt_end_notify	= iscsi_tm_done,
+	.transportid		= iscsi_transportid,
+	.default_bst		= "rdwr",
+};
+
 __attribute__((constructor)) static void iscsi_driver_constructor(void)
 {
 	register_driver(&iscsi);
