@@ -753,21 +753,13 @@ static void iser_task_free_in_bufs(struct iser_task *task)
 
 static void iser_complete_task(struct iser_task *task)
 {
+	struct iser_conn *conn = task->conn;
+
 	if (unlikely(task->opcode != ISCSI_OP_SCSI_CMD)) {
 		dprintf("task:%p, non-cmd\n", task);
 		return;
 	}
 
-	/* we are completing scsi cmd task, returning from target */
-	if (likely(task_in_scsi(task))) {
-		target_cmd_done(&task->scmd);
-		clear_task_in_scsi(task);
-	}
-	if (task->extdata) {
-		free(task->extdata);
-		task->extdata = NULL;
-	}
-
 	list_del(&task->session_list);
 	if (task->is_read)
 		iser_task_free_in_bufs(task);
@@ -775,7 +767,17 @@ static void iser_complete_task(struct iser_task *task)
 		iser_task_free_out_bufs(task);
 		/* iser_task_free_dout_tasks(task); // ToDo: multiple out buffers */
 	}
-	dprintf("task:%p\n", task);
+
+	/* we are completing scsi cmd task, returning from target */
+	if (likely(task_in_scsi(task))) {
+		target_cmd_done(&task->scmd);
+		clear_task_in_scsi(task);
+		iser_conn_put(conn);
+	}
+	if (task->extdata) {
+		free(task->extdata);
+		task->extdata = NULL;
+	}
 }
 
 static char *iser_conn_login_phase_name(enum iser_login_phase phase)
@@ -1215,6 +1217,16 @@ void iser_conn_close(struct iser_conn *conn)
 		&conn->h, conn->cm_id, conn->h.refcount);
 }
 
+static void iser_conn_force_close(struct iscsi_connection *iscsi_conn)
+{
+	struct iser_conn *conn = ISER_CONN(iscsi_conn);
+
+	eprintf("conn:%p\n", &conn->h);
+	conn->h.closed = 1;
+	iser_conn_close(conn);
+	iser_conn_put(conn);
+}
+
 /*
  * Called when the connection is freed, from iscsi, but won't do anything until
  * all posted WRs have gone away.  So also called again from RX progress when
@@ -1809,12 +1821,15 @@ static int iser_tm_exec(struct iser_task *task)
 					  (unsigned long) task, fn, req_bhs->lun,
 					  req_bhs->rtt, 0);
 		set_task_in_scsi(task);
+		iser_conn_get(conn);
+
 		switch (ret) {
 		case MGMT_REQ_QUEUED:
 			break;
 		case MGMT_REQ_FAILED:
 		case MGMT_REQ_DONE:
 			clear_task_in_scsi(task);
+			iser_conn_put(conn);
 			break;
 		}
 	}
@@ -1952,6 +1967,8 @@ static void iser_scsi_cmd_iosubmit(struct iser_task *task)
 	dprintf("task:%p tag:0x%04"PRIx64 "\n", task, task->tag);
 
 	set_task_in_scsi(task);
+	iser_conn_get(conn);
+
 	target_cmd_queue(session->target->tid, scmd);
 }
 
@@ -2158,8 +2175,7 @@ static void iser_sched_tx(struct event_data *evt)
 	int num_reqs = 0;
 	int err;
 
-	if (unlikely(conn->h.state == STATE_CLOSE ||
-		     conn->h.state == STATE_EXIT)) {
+	if (unlikely(conn->h.state == STATE_CLOSE)) {
 		dprintf("conn:%p closing, ignoring tx\n", conn);
 		return;
 	}
@@ -2788,9 +2804,6 @@ out:
 static void iser_login_tx_complete(struct iser_conn *conn)
 {
 	switch (conn->h.state) {
-	case STATE_EXIT:
-		conn->h.state = STATE_CLOSE;
-		break;
 	case STATE_SECURITY_LOGIN:
 		conn->h.state = STATE_LOGIN;
 		break;
@@ -3415,6 +3428,7 @@ static struct iscsi_transport iscsi_iser = {
 	.rdma   		= 1,
 	.data_padding   	= 1,
 	.ep_show		= iser_show,
+	.ep_force_close		= iser_conn_force_close,
 	.ep_getsockname 	= iser_getsockname,
 	.ep_getpeername 	= iser_getpeername,
 };
