@@ -513,7 +513,7 @@ int iscsi_target_update(int mode, int op, int tid, uint64_t sid, uint64_t lun,
 		break;
 	case MODE_CONNECTION:
 		if (op == OP_DELETE)
-			err = conn_close_force(tid, sid, cid);
+			err = conn_close_admin(tid, sid, cid);
 		break;
 	default:
 		break;
@@ -609,6 +609,60 @@ static int iscsi_target_show_stats(struct iscsi_target *target, uint64_t sid,
 
 }
 
+static int iscsi_target_show_connections(struct iscsi_target *target,
+					 uint64_t sid,
+					 char *buf, int rest)
+{
+	struct iscsi_session *session;
+	struct iscsi_connection *conn;
+	char addr[128];
+	int len = 0, total = 0;
+
+	list_for_each_entry(session, &target->sessions_list, slist) {
+		list_for_each_entry(conn, &session->conn_list, clist) {
+			memset(addr, 0, sizeof(addr));
+			conn->tp->ep_show(conn, addr, sizeof(addr));
+
+
+			len = snprintf(buf, rest, "Session: %u\n"
+				_TAB1 "Connection: %u\n"
+				_TAB2 "Initiator: %s\n"
+				_TAB2 "%s\n",
+				session->tsih,
+				conn->cid,
+				session->initiator,
+				addr);
+			__buffer_check(buf, total, len, rest);
+		}
+	}
+	return total;
+}
+
+static int iscsi_target_show_portals(struct iscsi_target *target, uint64_t sid,
+				   char *buf, int rest)
+{
+	int len = 0, total = 0;
+	struct iscsi_portal *portal;
+
+	list_for_each_entry(portal, &iscsi_portals_list,
+			iscsi_portal_siblings) {
+		int is_ipv6;
+
+		is_ipv6 = strchr(portal->addr, ':') != NULL;
+		len = snprintf(buf, rest,
+			       "Portal: %s%s%s:%d,%d\n",
+			       is_ipv6 ? "[" : "",
+			       portal->addr,
+			       is_ipv6 ? "]" : "",
+			       portal->port ? portal->port : ISCSI_LISTEN_PORT,
+			       portal->tpgt);
+		__buffer_check(buf, total, len, rest);
+	}
+
+	return total;
+
+}
+
 static int show_redirect_info(struct iscsi_target* target, char *buf, int rest)
 {
 	int len, total = 0;
@@ -647,7 +701,7 @@ int iscsi_target_show(int mode, int tid, uint64_t sid, uint32_t cid, uint64_t lu
 	struct iscsi_target* target = NULL;
 	int len, total = 0;
 
-	if (mode != MODE_SYSTEM) {
+	if (mode != MODE_SYSTEM && mode != MODE_PORTAL) {
 	    target = target_find_by_id(tid);
 	    if (!target)
 		    return -TGTADM_NO_TARGET;
@@ -670,6 +724,14 @@ int iscsi_target_show(int mode, int tid, uint64_t sid, uint32_t cid, uint64_t lu
 		len = iscsi_target_show_session(target, sid, buf, rest);
 		total += len;
 		break;
+	case MODE_PORTAL:
+		len = iscsi_target_show_portals(target, sid, buf, rest);
+		total += len;
+		break;
+	case MODE_CONNECTION:
+		len = iscsi_target_show_connections(target, sid, buf, rest);
+		total += len;
+		break;
 	case MODE_STATS:
 		len = iscsi_target_show_stats(target, sid, buf, rest);
 		total += len;
