@@ -59,6 +59,7 @@ struct mgmt_task {
 };
 
 static int ipc_fd;
+char mgmt_path[256];
 
 static void set_show_results(struct tgtadm_rsp *rsp, int *err)
 {
@@ -163,6 +164,41 @@ static int target_mgmt(int lld_no, struct mgmt_task *mtask)
 	return err;
 }
 
+static int portal_mgmt(int lld_no, struct mgmt_task *mtask,
+		       struct tgtadm_req *req,
+		       struct tgtadm_rsp *rsp)
+{
+	int err = TGTADM_INVALID_REQUEST;
+
+	switch (req->op) {
+	case OP_SHOW:
+		if (tgt_drivers[lld_no]->show) {
+			err = tgt_drivers[lld_no]->show(req->mode,
+							req->tid, req->sid,
+							req->cid, req->lun,
+							mtask->buf,
+							mtask->bsize);
+
+			set_show_results(rsp, &err);
+			return err;
+		}
+		break;
+	case OP_NEW:
+		err = tgt_portal_create(lld_no, mtask->buf);
+		break;
+	case OP_DELETE:
+		err = tgt_portal_destroy(lld_no, mtask->buf);
+		break;
+	default:
+		break;
+	}
+
+	rsp->err = err;
+	rsp->len = sizeof(*rsp);
+
+	return err;
+}
+
 static int device_mgmt(int lld_no, struct tgtadm_req *req, char *params,
 		       struct tgtadm_rsp *rsp, int *rlen)
 {
@@ -302,13 +338,24 @@ static int sys_mgmt(int lld_no, struct mgmt_task *mtask)
 	return err;
 }
 
-static int connection_mgmt(int lld_no, struct mgmt_task *mtask)
+static int connection_mgmt(int lld_no, struct mgmt_task *mtask,
+			   struct tgtadm_req *req,
+			   struct tgtadm_rsp *rsp)
 {
-	struct tgtadm_req *req = &mtask->req;
-	struct tgtadm_rsp *rsp = &mtask->rsp;
 	int err = TGTADM_INVALID_REQUEST;
 
 	switch (req->op) {
+	case OP_SHOW:
+		if (tgt_drivers[lld_no]->show) {
+			err = tgt_drivers[lld_no]->show(req->mode,
+							req->tid, req->sid,
+							req->cid, req->lun,
+							mtask->buf,
+							mtask->bsize);
+			set_show_results(rsp, &err);
+			return err;
+		}
+		break;
 	default:
 		if (tgt_drivers[lld_no]->update)
 			err = tgt_drivers[lld_no]->update(req->mode, req->op,
@@ -352,6 +399,9 @@ static int tgt_mgmt(struct mgmt_task *mtask)
 	case MODE_TARGET:
 		err = target_mgmt(lld_no, mtask);
 		break;
+	case MODE_PORTAL:
+		err = portal_mgmt(lld_no, mtask, req, rsp);
+		break;
 	case MODE_DEVICE:
 		err = device_mgmt(lld_no, req, mtask->buf, rsp, &len);
 		break;
@@ -359,7 +409,7 @@ static int tgt_mgmt(struct mgmt_task *mtask)
 		err = account_mgmt(lld_no, mtask);
 		break;
 	case MODE_CONNECTION:
-		err = connection_mgmt(lld_no, mtask);
+		err = connection_mgmt(lld_no, mtask, req, rsp);
 		break;
 	default:
 		if (req->op == OP_SHOW && tgt_drivers[lld_no]->show) {
@@ -567,7 +617,6 @@ int ipc_init(void)
 	extern short control_port;
 	int fd, err;
 	struct sockaddr_un addr;
-	char path[256];
 
 	fd = socket(AF_LOCAL, SOCK_STREAM, 0);
 	if (fd < 0) {
@@ -575,11 +624,11 @@ int ipc_init(void)
 		return -1;
 	}
 
-	sprintf(path, "%s.%d", TGT_IPC_NAMESPACE, control_port);
-	unlink(path);
+	sprintf(mgmt_path, "%s.%d", TGT_IPC_NAMESPACE, control_port);
+	unlink(mgmt_path);
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_LOCAL;
-	strncpy(addr.sun_path, path, sizeof(addr.sun_path));
+	strncpy(addr.sun_path, mgmt_path, sizeof(addr.sun_path));
 
 	err = bind(fd, (struct sockaddr *) &addr, sizeof(addr));
 	if (err) {
