@@ -1061,7 +1061,7 @@ static void __cmd_done(struct target *target, struct scsi_cmd *cmd)
 
 	err = target->bst->bs_cmd_done(cmd);
 
-	dprintf("%d %p %p %u %u %d\n", cmd_mmapio(cmd), scsi_get_out_buffer(cmd),
+	dprintf("%p %p %u %u %d\n", scsi_get_out_buffer(cmd),
 		scsi_get_in_buffer(cmd), scsi_get_out_length(cmd),
 		scsi_get_in_length(cmd), err);
 
@@ -1086,7 +1086,7 @@ void __cmd_done_passthrough(struct target *target, struct scsi_cmd *cmd)
 
 	err = target->bst->bs_cmd_done(cmd);
 
-	dprintf("%d %p %p %u %u %d\n", cmd_mmapio(cmd), scsi_get_out_buffer(cmd),
+	dprintf("%p %p %u %u %d\n", scsi_get_out_buffer(cmd),
 		scsi_get_in_buffer(cmd), scsi_get_out_length(cmd),
 		scsi_get_in_length(cmd), err);
 }
@@ -1795,7 +1795,8 @@ int tgt_target_show_all(char *buf, int rest)
 				 lu->path ? : "None",
 				 open_flags_to_str(strflags, lu->bsoflags));
 
-		if (!strcmp(tgt_drivers[target->lid]->name, "iscsi")) {
+		if (!strcmp(tgt_drivers[target->lid]->name, "iscsi") ||
+		    !strcmp(tgt_drivers[target->lid]->name, "iser")) {
 			int i, aid;
 
 			shprintf(total, buf, rest, _TAB1
@@ -1963,6 +1964,56 @@ int tgt_target_destroy(int lld_no, int tid)
 	return 0;
 }
 
+int tgt_portal_create(int lld, char *args)
+{
+	char *portals = NULL;
+
+	portals = strstr(args, "portal=");
+	if (!portals) {
+		eprintf("invalid option when creating portals: %s\n", args);
+		return TGTADM_INVALID_REQUEST;
+	}
+
+	if (tgt_drivers[lld]->portal_create) {
+		if (tgt_drivers[lld]->portal_create(portals)) {
+			eprintf("failed to create portal %s\n", portals);
+			return TGTADM_INVALID_REQUEST;
+		}
+	} else {
+		eprintf("can not create portals for for this lld type\n");
+		return TGTADM_INVALID_REQUEST;
+	}
+
+	dprintf("succeed to create new portals %s\n", portals);
+
+	return 0;
+}
+
+int tgt_portal_destroy(int lld, char *args)
+{
+	char *portals = NULL;
+
+	portals = strstr(args, "portal=");
+	if (!portals) {
+		eprintf("invalid option when destroying portals: %s\n", args);
+		return TGTADM_INVALID_REQUEST;
+	}
+
+	if (tgt_drivers[lld]->portal_destroy) {
+		if (tgt_drivers[lld]->portal_destroy(portals)) {
+			eprintf("failed to destroy portal %s\n", portals);
+			return TGTADM_INVALID_REQUEST;
+		}
+	} else {
+		eprintf("can not destroy portals for for this lld type\n");
+		return TGTADM_INVALID_REQUEST;
+	}
+
+	dprintf("succeed to destroy portals %s\n", portals);
+
+	return 0;
+}
+
 int account_show(char *buf, int rest)
 {
 	int total = 0, max = rest;
@@ -1987,7 +2038,7 @@ static struct {
 	{TGT_SYSTEM_READY, "ready"},
 };
 
-static char *system_state_name(enum scsi_target_state state)
+static char *system_state_name(enum tgt_system_state state)
 {
 	int i;
 	char *name = NULL;
