@@ -285,27 +285,68 @@ int call_program(const char *cmd, void (*callback)(void *data, int result),
 	argv[i] =  NULL;
 
 	ret = pipe(fds);
-	if (ret < 0)
+	if (ret < 0) {
+		eprintf("pipe create failed for %s, %m\n", cmd);
 		return ret;
+	}
 
-	eprintf("%d %d\n", fds[0], fds[1]);
+	dprintf("%s, pipe: %d %d\n", cmd, fds[0], fds[1]);
 
 	pid = fork();
-	if (pid < 0)
+	if (pid < 0) {
+		eprintf("fork failed for: %s, %m\n", cmd);
+		close(fds[0]);
+		close(fds[1]);
 		return pid;
+	}
 
 	if (!pid) {
 		close(1);
-		dup(fds[1]);
+		ret = dup(fds[1]);
+		if (ret < 0) {
+			eprintf("dup failed for: %s, %m\n", cmd);
+			exit(-1);
+		}
 		close(fds[0]);
-		ret = execv(argv[0], argv);
+		execv(argv[0], argv);
+
+		eprintf("execv failed for: %s, %m\n", cmd);
 		exit(-1);
 	} else {
+		struct timeval tv;
+		fd_set rfds;
+		int ret_sel;
+
 		close(fds[1]);
-		waitpid(pid, &i, 0);
-		ret = read(fds[0], output, op_len);
-		if (ret < 0)
-			eprintf("failed to get the output from <%s>.", cmd);
+		/* 0.1 second is okay, as the initiator will retry anyway */
+		do {
+			FD_ZERO(&rfds);
+			FD_SET(fds[0], &rfds);
+			tv.tv_sec = 0;
+			tv.tv_usec = 100000;
+			ret_sel = select(fds[0]+1, &rfds, NULL, NULL, &tv);
+		} while (ret_sel < 0 && errno == EINTR);
+		if (ret_sel <= 0) { /* error or timeout */
+			eprintf("timeout on redirect callback, terminating "
+				"child pid %d\n", pid);
+			kill(pid, SIGTERM);
+		}
+		do {
+			ret = waitpid(pid, &i, 0);
+		} while (ret < 0 && errno == EINTR);
+		if (ret < 0) {
+			eprintf("waitpid failed for: %s, %m\n", cmd);
+			close(fds[0]);
+			return ret;
+		}
+		if (ret_sel > 0) {
+			ret = read(fds[0], output, op_len);
+			if (ret < 0) {
+				eprintf("failed to get output from: %s\n", cmd);
+				close(fds[0]);
+				return ret;
+			}
+		}
 
 		if (callback)
 			callback(data, WEXITSTATUS(i));
