@@ -19,6 +19,7 @@
  *
  */
 
+#define _XOPEN_SOURCE 600
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -38,19 +39,8 @@
 #include "libssc.h"
 #include "scsi.h"
 
-#undef eprintf
-#define eprintf(fmt, args...)						\
-do {									\
-	fprintf(stderr, "%s: " fmt, program_name, ##args);		\
-} while (0)
-
-#undef dprintf
-#define dprintf(fmt, args...)						\
-do {									\
-	if (debug)							\
-		fprintf(stderr, "%s %d: " fmt,				\
-			__FUNCTION__, __LINE__, ##args);		\
-} while (0)
+#define NO_LOGGING
+#include "log.h"
 
 enum {
 	OP_NEW,
@@ -429,7 +419,7 @@ static int sbc_new(int op, char *path, char *capacity, char *media_type)
 	int fd;
 
 	if (!strncasecmp("disk", media_type, 4)) {
-		uint32_t pos, size;
+		uint32_t size;
 		char *buf;
 
 		sscanf(capacity, "%d", &size);
@@ -448,9 +438,10 @@ static int sbc_new(int op, char *path, char *capacity, char *media_type)
 			perror("Failed creating file");
 			exit(2);
 		}
-
-		for (pos = 0; pos < size; pos++)
-			pwrite(fd, buf, 1024*1024, pos*1024*1024LL);
+		if (posix_fallocate(fd, 0, size*1024*1024LL) == -1) {
+			perror("posix_fallocate failed.");
+			exit(3);
+		}
 
 		free(buf);
 		close(fd);
